@@ -17,6 +17,15 @@ CMD=bash
 ROOT=$(realpath $(dirname $0)/..)
 cd $ROOT
 
+sudo modprobe openvswitch
+sudo modprobe ebtables
+
+if which apparmor_status >&/dev/null ; then
+    if sudo apparmor_status --enabled ; then
+        sudo apparmor_parser -R /etc/apparmor.d/usr.sbin.tcpdump || :
+    fi
+fi
+
 sudo docker build --pull -t faucet/tests -f Dockerfile.tests  .
 
 echo
@@ -24,10 +33,16 @@ echo "environment set:"
 echo "  FAUCET_TESTS=\"$FAUCET_TESTS\""
 echo "try:"
 echo "  docker/runtests.sh"
-echo "or:"
-echo '  cd tests && ./run_integration_tests.sh $FAUCET_TESTS'
-echo '  cd clib && ./clib_mininet_test.py $FAUCET_TESTS'
 echo
 
-sudo docker run -ti -v $PWD:/faucet-src -e FAUCET_TESTS="$FAUCET_TESTS" \
-       --sysctl net.ipv6.conf.all.disable_ipv6=0 --privileged faucet/tests $CMD
+mkdir -p test_results
+mkdir -p /tmp/faucet-pip-cache
+
+sudo docker run -ti \
+     --privileged \
+     --sysctl net.ipv6.conf.all.disable_ipv6=0 \
+     -v $PWD:/faucet-src \
+     -v $PWD/test_results:/var/tmp \
+     -v /tmp/faucet-pip-cache:/var/tmp/pip-cache \
+     -e FAUCET_TESTS="$FAUCET_TESTS" \
+     faucet/tests $CMD
