@@ -489,6 +489,7 @@ configuration.
                 included_tables.add('vip')
         if valve_cl.STATIC_TABLE_IDS:
             included_tables.add('port_acl')
+            self.has_acls = True
         if self.hairpin_ports:
             included_tables.add('eth_dst_hairpin')
         if self.use_classification:
@@ -805,6 +806,14 @@ configuration.
         """Remove a stack link to the stack graph."""
         return cls.modify_stack_topology(graph, dp, port, False)
 
+    @staticmethod
+    def hash_graph(graph):
+        """Return hash of a topology graph"""
+        # Using the degree of the topology is a quick way to get an estimate on
+        #   whether a graph is isomorphic which is what we would really want when comparing
+        #   two graphs.
+        return hash(tuple(sorted(graph.degree())))
+
     def resolve_stack_topology(self, dps, meta_dp_state):
         """Resolve inter-DP config for stacking."""
         stack_dps = [dp for dp in dps if dp.stack is not None]
@@ -1322,6 +1331,12 @@ configuration.
             port for port in self.ports.values() if port.loop_protect_external}
         self.has_externals = bool(loop_protect_external_ports)
 
+        # Populate port.lacp_port_id if it wasn't set in config
+        for port in self.ports.values():
+            if port.lacp and port.lacp_port_id == -1:
+                dp_index = dps.index(self)
+                port.lacp_port_id = dp_index * 100 + port.number
+
         resolve_stack_dps()
         resolve_mirror_destinations()
         resolve_acls()
@@ -1465,6 +1480,19 @@ configuration.
         changed_acl_ports = set()
         all_ports_changed = False
 
+        topology_changed = False
+        if self.stack_graph:
+            topology_changed = bool(self.hash_graph(self.stack_graph) != self.hash_graph(new_dp.stack_graph))
+        if topology_changed:
+            # Topology changed so restart stack ports just to be safe
+            stack_ports = [
+                port.number for port in new_dp.stack_ports
+                if port.number not in deleted_ports and
+                port.number not in added_ports]
+            changed_ports.update(set(stack_ports))
+            logger.info('Stack topology change detected, restarting stack ports')
+            same_ports -= changed_ports
+
         if not same_ports:
             all_ports_changed = True
         # TODO: optimize case where only VLAN ACL changed.
@@ -1604,7 +1632,9 @@ configuration.
                 deleted_vlans (set): deleted VLAN IDs.
                 changed_vlans (set): changed/added VLAN IDs.
                 all_ports_changed (bool): True if all ports changed.
+                all_meters_changed (bool): True if all meters changed
                 deleted_meters (set): deleted meter numbers
+                added_meters (set): Added meter numbers
                 changed_meters (set): changed/added meter numbers
         """
         if new_dp._table_configs() != self._table_configs():
