@@ -26,6 +26,14 @@ STACK_STATE_BAD = 2
 STACK_STATE_UP = 3
 STACK_STATE_GONE = 4
 STACK_STATE_NONE = -1
+STACK_DISPLAY_DICT = {
+    STACK_STATE_ADMIN_DOWN: 'ADMIN_DOWN',
+    STACK_STATE_INIT: 'INITIALIZING',
+    STACK_STATE_BAD: 'BAD',
+    STACK_STATE_UP: 'UP',
+    STACK_STATE_GONE: 'GONE',
+    STACK_STATE_NONE: 'NONE'
+}
 
 # LACP not configured
 LACP_ACTOR_NOTCONFIGURED = -1
@@ -107,6 +115,8 @@ class Port(Conf):
         # Min time since last LACP response. Used to control rate of response for LACP
         'lacp_port_priority': 255,
         # Sets port priority value sent out in lacp packet
+        'lacp_port_id': -1,
+        # Sets port id value sent out in lacp packet
         'loop_protect': False,
         # if True, do simple (host/access port) loop protection on this port.
         'loop_protect_external': False,
@@ -164,6 +174,7 @@ class Port(Conf):
         'lacp_passthrough': list,
         'lacp_resp_interval': int,
         'lacp_port_priority': int,
+        'lacp_port_id': int,
         'loop_protect': bool,
         'loop_protect_external': bool,
         'output_only': bool,
@@ -225,6 +236,7 @@ class Port(Conf):
         self.lacp_passthrough = None
         self.lacp_resp_interval = None
         self.lacp_port_priority = None
+        self.lacp_port_id = None
         self.loop_protect = None
         self.loop_protect_external = None
         self.max_hosts = None
@@ -682,3 +694,7 @@ class Port(Conf):
     def stack_gone(self):
         """Change the current stack state to GONE."""
         self.dyn_stack_current_state = STACK_STATE_GONE
+
+    def stack_state_name(self, state):
+        """Return stack state name"""
+        return STACK_DISPLAY_DICT[state]
