@@ -3,7 +3,7 @@
 # Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
-# Copyright (C) 2015--2018 The Contributors
+# Copyright (C) 2015--2019 The Contributors
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,12 +38,15 @@ class ValvePipeline(ValveManagerBase):
         self.classification_table = dp.classification_table()
         self.output_table = dp.output_table()
         self.egress_table = None
+        self.egress_acl_table = None
         if dp.egress_pipeline:
             self.egress_table = dp.tables['egress']
+            self.egress_acl_table = dp.tables.get('egress_acl')
         self.filter_priority = self._FILTER_PRIORITY
         self.select_priority = self._HIGH_PRIORITY
 
-    def _accept_to_table(self, table, actions):
+    @staticmethod
+    def _accept_to_table(table, actions):
         inst = [table.goto_this()]
         if actions is not None:
             inst.append(valve_of.apply_actions(actions))
@@ -79,7 +82,21 @@ class ValvePipeline(ValveManagerBase):
         """
         return self._accept_to_table(self.output_table, actions)
 
-    def output(self, port, vlan, hairpin=False, loop_protect_field=None):
+    def accept_to_egress(self, actions=None):
+        """Get instructions to forward packet through the pipeline to egress
+        table
+
+        Raises an assertion error if egress pipeline is not configured
+
+        args:
+            actions: (optional) list of actions to apply to the packet
+        returns:
+            list of instructions:
+        """
+        assert self.egress_table is not None
+        return self._accept_to_table(self.egress_table, actions)
+
+    def output(self, port, vlan, hairpin=False, external_forwarding_requested=None):
         """Get instructions list to output a packet through the regular
         pipeline.
 
@@ -87,6 +104,8 @@ class ValvePipeline(ValveManagerBase):
             port: Port object of port to output packet to
             vlan: Vlan object of vlan to output packet on
             hairpin: if True, hairpinning is required
+            apply_egress_acl: if True the packet will be sent to the egress acl
+                table before being output
         returns:
             list of Instructions
         """
@@ -94,12 +113,16 @@ class ValvePipeline(ValveManagerBase):
         if self.egress_table:
             metadata, metadata_mask = faucet_md.get_egress_metadata(
                 port.number, vlan.vid)
-            instructions.extend(valve_of.metadata_goto_table(
-                metadata, metadata_mask, self.egress_table))
+            if self.egress_acl_table:
+                instructions.extend(valve_of.metadata_goto_table(
+                    metadata, metadata_mask, self.egress_acl_table))
+            else:
+                instructions.extend(valve_of.metadata_goto_table(
+                    metadata, metadata_mask, self.egress_table))
         else:
             instructions.append(valve_of.apply_actions(vlan.output_port(
                 port, hairpin=hairpin, output_table=self.output_table,
-                loop_protect_field=loop_protect_field)))
+                external_forwarding_requested=external_forwarding_requested)))
         return instructions
 
     def initialise_tables(self):
@@ -117,9 +140,17 @@ class ValvePipeline(ValveManagerBase):
         # antispoof for FAUCET's MAC address
         # TODO: antispoof for controller IPs on this VLAN, too.
         if self.dp.drop_spoofed_faucet_mac:
-            for vlan in list(self.dp.vlans.values()):
-                ofmsgs.extend(self.filter_packets(
-                    {'eth_src': vlan.faucet_mac}))
+            if self.dp.stack_ports:
+                vlan_macs = {vlan.faucet_mac for vlan in self.dp.vlans.values()}
+                for port in self.dp.ports.values():
+                    if not port.stack:
+                        for mac in vlan_macs:
+                            ofmsgs.extend(self.filter_packets(
+                                {'eth_src': mac, 'in_port': port.number}))
+            else:
+                for vlan in list(self.dp.vlans.values()):
+                    ofmsgs.extend(self.filter_packets(
+                        {'eth_src': vlan.faucet_mac}))
 
         return ofmsgs
 
