@@ -115,7 +115,7 @@ class VLAN(Conf):
         # If True, forward packets from the VLAN table to the VLAN_ACL table matching the VID
         'dot1x_assigned': False,
         # If True, this VLAN may be dynamically added withTunnel-Private-Group-ID radius attribute.
-        'edge_learn_stack_root': False,
+        'edge_learn_stack_root': True,
         # If True, this VLAN will learn flows through the stack root, following forwarding path.
         }
 
